/*
 * Copyright 2024 NXP
 *
 * SPDX-License-Identifier:     BSD-3-Clause
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

long long current_timestamp() {
    struct timeval te;
    gettimeofday(&te, NULL); // get current time
    long long milliseconds = te.tv_sec*1000LL + te.tv_usec/1000; // calculate milliseconds
    return milliseconds;
}

void init(int *arr, int size) {
  for(int i = 0; i < size; i++) {
    do {
      arr[i] = rand() % size;
    } while(arr[i] == i);
  }
}

int run(int *arr, int size, int iter) {
  int rez = 0;
  for(int g = 0; g < iter; g++)
    for(int i = 0; i < size; i++) {
      rez += arr[arr[arr[arr[arr[i]]]]];
    }
  return rez;
}

volatile int tmp;

int main(int argc, char *argv[]) {
  if(argc < 4) {
    printf("\nUsage: usecache mem_size load ID\n");
    printf(" mem_size in KiB\n load - no. of 4-byte random memory accesses = load*5120000\n ID - no. to be printed at the end\n\n");
    return -1;
  }
  int size = atoi(argv[1])*1024/sizeof(int);
  int iter = atoi(argv[2])*32*32/atoi(argv[1]);
  int id = atoi(argv[3]);

  printf("Running %d iterations on mem_size %ld KiB, ID: %d\n",
          iter, size*sizeof(int)/1024, id);

  int *a = malloc(size*sizeof(int));
  if(!a) {
    printf("Not enough memory\n");
    return -1;
  }

  init(a, size);
  long long start = current_timestamp();
  tmp = run(a, size, iter);
  printf("ID: %d Time: %lld milliseconds\n", id, current_timestamp()-start);

  return 0;
}
